//
//  Author(s)...: ATMEL Norway
//
//  Target(s)...: ATmega169
//
//  Redigert: Martin Korsgaard 24.01.07

// Include files.
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <avr/interrupt.h>

#include "macro.h"
#include "lcd_driver.h"

// Constants
#define LCD_TIMER_SEED		    3
#define LCD_REGISTER_COUNT      20
#define TEXTBUFFER_SIZE         30

// FOR USE BY STDIO
static int lcd_putchar(char c);
FILE *lcd = NULL;

// DEVICE SPECIFIC!!! (ATmega169)
#define PLCDREG ((unsigned char *)(0xEC))

// DEVICE SPECIFIC!!! (ATmega169) First LCD segment register
#define LCD_CONTRAST_LEVEL(level) LCDCCR=(0x0F & level)

// Used to indicate when the LCD interrupt handler should update the LCD
static volatile char lcd_update_required_g = FALSE;

// LCD display buffer (for double buffering).
static char lcd_data_g[LCD_REGISTER_COUNT];

// Buffer that contains the text to be displayed
static char lcd_text_buffer_g[TEXTBUFFER_SIZE] = "";

// Only six letters can be shown on the LCD.
// With the lcd_scroll_g and lcd_scroll_mode_g variables,
// one can select which part of the buffer to show
static volatile signed char lcd_scroll_g;
static volatile char lcd_scroll_mode_g;

////Start-up delay before scrolling a string over the LCD
static char lcd_start_scroll_timer_g = 0;

// Look-up table used when converting ASCII to
// LCD display data (segment control)
static const unsigned int lcd_character_table_g[] PROGMEM = {
	0x0A51,     // '*' (?)
	0x2A80,     // '+'
	0x0000,     // ',' (Not defined)
	0x0A00,     // '-'
	0x0A51,     // '.' Degree sign
	0x0000,     // '/' (Not defined)
	0x5559,     // '0'
	0x0118,     // '1'
	0x1e11,     // '2
	0x1b11,     // '3
	0x0b50,     // '4
	0x1b41,     // '5
	0x1f41,     // '6
	0x0111,     // '7
	0x1f51,     // '8
	0x1b51,     // '9'
	0x0000,     // ':' (Not defined)
	0x0000,     // ';' (Not defined)
	0x0000,     // '<' (Not defined)
	0x0000,     // '=' (Not defined)
	0x0000,     // '>' (Not defined)
	0x0000,     // '?' (Not defined)
	0x0000,     // '@' (Not defined)
    0x0f51,     // 'A' (+ 'a')
    0x3991,     // 'B' (+ 'b')
    0x1441,     // 'C' (+ 'c')
    0x3191,     // 'D' (+ 'd')
    0x1e41,     // 'E' (+ 'e')
    0x0e41,     // 'F' (+ 'f')
    0x1d41,     // 'G' (+ 'g')
    0x0f50,     // 'H' (+ 'h')
    0x2080,     // 'I' (+ 'i')
    0x1510,     // 'J' (+ 'j')
    0x8648,     // 'K' (+ 'k')
    0x1440,     // 'L' (+ 'l')
    0x0578,     // 'M' (+ 'm')
    0x8570,     // 'N' (+ 'n')
    0x1551,     // 'O' (+ 'o')
    0x0e51,     // 'P' (+ 'p')
    0x9551,     // 'Q' (+ 'q')
    0x8e51,     // 'R' (+ 'r')
    0x9021,     // 'S' (+ 's')
    0x2081,     // 'T' (+ 't')
    0x1550,     // 'U' (+ 'u')
    0x4448,     // 'V' (+ 'v')
    0xc550,     // 'W' (+ 'w')
    0xc028,     // 'X' (+ 'x')
    0x2028,     // 'Y' (+ 'y')
    0x5009,     // 'Z' (+ 'z')
    0x0000,     // '[' (Not defined)
    0x0000,     // '\' (Not defined)
    0x0000,     // ']' (Not defined)
    0x0000,     // '^' (Not defined)
    0x0000      // '_'
};



static int lcd_putchar(char c)
{
	static uint8_t i = 0;
	
	// Wait for access to buffer
	while (lcd_update_required_g);

	// When encountering eol:
	if (c == '\n' || c == '\r' || c == '\0') {
		// Terminate buffer string
		lcd_text_buffer_g[i] = '\0';

		// Reset scrolling, enable it if string is too long to fit at once.
		lcd_scroll_g = 0;
		lcd_scroll_mode_g = (i > 6);

		// Signal that an update is required, reset position pointer.
		lcd_update_required_g = 1;
		i = 0;

		return '\0';
	}
	else {
		// If too many digits to fit in buffer: Ignore
		if (i >= TEXTBUFFER_SIZE-1)
			return '\0';
		
		// Add new character to buffer.
		lcd_text_buffer_g[i++] = c;
		return c;
	}
}



/*****************************************************************************
*
*   Function name : lcd_all_segments(unsigned char input)
*
*   Returns :       None
*
*   Parameters :    show -  [TRUE;FALSE]
*
*   Purpose :       shows or hide all all LCD segments on the LCD
*
*****************************************************************************/
static void lcd_all_segments(char show)
{
    unsigned char i;

    if (show)
        show = 0xFF;

    // Set/clear all bits in all LCD registers
    for (i=0; i < LCD_REGISTER_COUNT; i++)
        *(lcd_data_g + i) = show;
}


/*****************************************************************************
*
*   Function name : LCD_Init
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Initialize LCD_displayData buffer.
*                   Set up the LCD (timing, contrast, etc.)
*
*****************************************************************************/
void lcd_init (void)
{
    lcd_all_segments(FALSE);                     // Clear segment buffer.

	// Set the LCD contrast level
	LCDCCR=0x0F;

    // Select asynchronous clock source, enable all COM pins and enable all
    // segment pins.
    LCDCRB = (1<<LCDCS) | (3<<LCDMUX0) | (7<<LCDPM0);

    // Set LCD prescaler to give a framerate of 32,0 Hz
    LCDFRR = (0<<LCDPS0) | (7<<LCDCD0);    

	// Enable LCD and set low power waveform
    LCDCRA = (1<<LCDEN) | (1<<LCDAB);

    //Enable LCD start of frame interrupt
    LCDCRA |= (1<<LCDIE);

	//Turn off colons
	*(PLCDREG + 8) = 0x00;

    lcd_update_required_g = FALSE;

	lcd = fdevopen(lcd_putchar, NULL);
}


/*****************************************************************************
*
*   Function name : lcd_write_digit(char c, char digit)
*
*   Returns :       None
*
*   Parameters :    Inputs
*                   c: The symbol to be displayed in a LCD digit
*                   digit: In which digit (0-5) the symbol should be displayed
*                   Note: Digit 0 is the first used digit on the LCD,
*                   i.e LCD digit 2
*
*   Purpose :       Stores LCD control data in the LCD_displayData buffer.
*                   (The LCD_displayData is latched in the LCD_SOF interrupt.)
*
*****************************************************************************/
static void lcd_write_digit(char c, char digit)
{
    unsigned int seg = 0x0000;                  // Holds the segment pattern
    char mask, nibble;
    char *ptr;
    char i;

    if (digit > 5)                              // Skip if digit is illegal
        return;

    //Lookup character table for segmet data
    if ((c >= '*') && (c <= 'z')) {
        // c is a letter, convert to uppercase if necessary.
        if (c >= 'a')
            c &= ~0x20;

        c -= '*';

		seg = (unsigned int) pgm_read_word(&lcd_character_table_g[(uint8_t)c]); 
	}

    // Adjust mask according to LCD segment mapping
    if (digit & 0x01)
		// Digit 1, 3, 5
        mask = 0x0F;
    else
		// Digit 0, 2, 4
        mask = 0xF0;

	// digit = {0,0,1,1,2,2}
    ptr = lcd_data_g + (digit >> 1);

    for (i = 0; i < 4; i++) {
        nibble = seg & 0x000F;
        seg >>= 4;
        if (digit & 0x01)
            nibble <<= 4;
        *ptr = (*ptr & mask) | nibble;
        ptr += 5;
    }
}



/*****************************************************************************
*
*   LCD Interrupt Routine
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose: Latch the LCD_displayData and Set LCD_status.updateComplete
*
*****************************************************************************/

SIGNAL(SIG_LCD)
{
    static char lcd_timer = LCD_TIMER_SEED;
    char c;
    char eol;
    unsigned char i;

    lcd_timer--; // Decreased every LCD frame

    if (lcd_scroll_mode_g) {
        // If we are in scroll mode, and the timer has expired,
        // we will update the LCD
        if (lcd_timer == 0) {
            if (lcd_start_scroll_timer_g == 0)
                lcd_update_required_g = TRUE;
            else
                lcd_start_scroll_timer_g--;
        }
    }
    else {   
		// if not scrolling, disble LCD start of frame interrupt
        lcd_scroll_g = 0;
    }

    eol = FALSE;
    if (lcd_update_required_g == TRUE) {
        // Repeat for the six LCD characters
        for (i = 0; i < 6; i++) {
            if ((lcd_scroll_g+i) >= 0 && (!eol)) {
                // We have some visible characters
                c = lcd_text_buffer_g[i + lcd_scroll_g];
                c = c & 0x7F;

                if (c == '\0')
					// End of character data
                    eol = i+1;
            }
            else
                c = ' ';

			lcd_write_digit(c, i);
        }

        // Copy the segment buffer to the real segments
        for (i = 0; i < LCD_REGISTER_COUNT; i++)
            *(PLCDREG + i) = *(lcd_data_g+i);

        // If the text scrolled off the display,
        // we have to start over again.
        if (eol == 1)
            lcd_scroll_g = -6;
        else
            lcd_scroll_g++;

        // No need to update anymore
        lcd_update_required_g = FALSE;
    }

    // lcd_timer is used when scrolling text
    if (lcd_timer == 0) {
		lcd_timer = LCD_TIMER_SEED;
    }
}

