#include <stdio.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <avr/io.h>

/*************************************************************************
* description:  interrupt driven uartdriver for at90can
**************************************************************************/

/************************************************************************/
/*                                                                      */
/*                      Interrupt driven buffered UART                  */
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*           editet by: Anders Guldahl    12/2009                       */
/************************************************************************/


#include <avr/interrupt.h>
#include "macro.h"
#include "uart_motor.h"

static volatile unsigned char rx_buff[RX0_SIZE];
static volatile unsigned char rx_in;
static volatile unsigned char rx_out;
static volatile unsigned char tx_buff[TX0_SIZE];
static volatile unsigned char tx_in;
static volatile unsigned char tx_out;


// count up and wrap around
#define ROLLOVER( x, max )	x = ++x >= max ? 0 : x


void USART_Init(){
	unsigned int brr;


	brr = BAUDREG_motor;

	/* Set baud rate */
	UBRR1H = (unsigned char)(brr>>8);
	UBRR1L = (unsigned char)brr;

	
	UCSR1A |= (1<<U2X1);
	/* Enable receiver and transmitter and rx-interrupt */
	UCSR1B = (1<<RXEN1)|(1<<TXEN1);

	/* Set frame format: 8data, 1stop bit */
	UCSR1C = (3<<UCSZ10);

	rx_in = 0;				// set buffers empty
  	rx_out = 0;
  	tx_in = 0;
  	tx_out = 0;

}

//0 = rx buffer empty
unsigned char ukbhit0( void )
{
  return rx_out ^ rx_in;		// rx_in modified by interrupt !
}

uint16_t ugetchar0( void )
{
  unsigned char data;

  if( !ukbhit0()){ // return uart_no_data if no new char arrived
  	return UART_NO_DATA;
  }
  data = rx_buff[rx_out];		// get byte
  ROLLOVER( rx_out, RX0_SIZE );
  UCSR1B |= (1<<RXCIE1);				// enable RX interrupt
  return data;
}

ISR( USART1_RX_vect )
{
  unsigned char i = rx_in;

  ROLLOVER( i, RX0_SIZE );
  if( i == rx_out ){			// buffer overflow
    UCSR1B &= ~(1<<RXCIE1);		// disable RX interrupt
    return;
  }
  rx_buff[rx_in] = UDR1;
  rx_in = i;
}


ISR( USART1_UDRE_vect )
{
  if( tx_in == tx_out ){		// nothing to sent
    UCSR1B &= ~(1<<UDRIE1);		// disable TX interrupt
    return;
  }
  UDR1 = tx_buff[tx_out];
  ROLLOVER( tx_out, TX0_SIZE );
}


unsigned char utx0_ready( void )
{
  unsigned char i = tx_in;

  ROLLOVER( i, TX0_SIZE );
  return tx_out ^ i;		// 0 = busy
}



void uputchar( unsigned char c )
{

	UDR1 = c;	// Send byte
	//_DELAY_MS(1);
	while ( !( UCSR1A & (1<<UDRE1)) ); // Wait for empty transmit buffer

/*
  unsigned char i = tx_in;

  ROLLOVER( i, TX0_SIZE );
  tx_buff[tx_in] = c;
  while( i == tx_out);		// until at least one byte free
							// tx_out modified by interrupt !
  tx_in = i;
  UCSR1B |= (1<<UDRIE1);        // enable TX interrupt
  */
}


void uputs( unsigned char *s )
{
  while( *s )
    uputchar( *s++ );
}

void uart_puts_P(const char *s) {
	while (pgm_read_byte(s) != 0x00) {
		uputchar(pgm_read_byte(s++));
	}
}



int ugets(unsigned char *buffer, int size){

	int i;

	for(i = 0;i<size;i++){
		if(ukbhit0() != 0){
			buffer[i] = ugetchar0();
		}
		else{
			return i;
		}
	}

	return i;


}


void USART_rx_flush( void )
{

	while(ugetchar0() != UART_NO_DATA);

}

